// Copyright (c) 2023 Citrix Systems, Inc. All Rights Reserved.
// You may only reproduce, distribute, perform, display, or prepare derivative works of this file pursuant to a valid license from Citrix.
// THIS SAMPLE CODE IS PROVIDED BY CITRIX "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

//
// CertificateIdentityDeclaration.cpp : Defines the entry points for the DLL.
//

#include <stdio.h>
#include <cstdlib>
#include <cstring>
#include <vector>

#include "CitrixAuthManagerCustomProtocolSDK.h"
#include "PostCertificateAndTokenRequest.h"
#include "Utilities.h"

using namespace std;

namespace CitrixAuthManagerCustomProtocolSDK
{

// This called by Auth Manager with the token request to pass to the given server URL.
// This function implements the client side of the protocol communicating with the paired
// StoreFront extension to authenticate the user and get back a StoreFront authentication
// token.
eAuthResult CitrixAuthenticationManager_Authenticate(UTF8ByteArray startURL,
                                                     UTF8ByteArray tokenRequest,
                                                     eAuthFlags flags, /* This passes in whether the plugin may show UI. This example does not use UI. */
                                                     const AMWindowInfo&, /* clientWindowInfo. This is unused in this example. It allows you to identify the client window. */
                                                     /* out */ UTF8ByteArray* outAuthenticationData,
                                                     /* out */ size_t* outDataSize)
{
    TRACE("Plugin's Authenticate function invoked");

    // These pointers should always be valid.
    if (startURL == NULL ||
            tokenRequest == NULL ||
            outAuthenticationData == NULL ||
            outDataSize == NULL)
    {
        TRACE("NULL pointer passed in from Auth Manager");

        return SystemError;
    }

    // Initialize the output parameters.
    *outAuthenticationData = NULL;
    *outDataSize = 0;

    // Note that in this example there is no UI to be shown because the plugin has no UI.
    // The interactive flag is specified by the client which has called Auth Manager to
    // make an HTTP request. That request has resulted in a need to authenticate to StoreFront.
    if (flags & Interactive)
    {
        TRACE("The plugin may show UI");
    }
    else
    {
        TRACE("The plugin may NOT show UI");
        // A background operation has caused the authentication to occur and is allowed & expected to fail if
        // the plugin cannot authenticate without showing UI.

        // If your plugin requires UI to complete and it is not allowed to be shown then return that
        // authentication failed because interaction was required.
        // return InteractionNotAllowed;
    }

    // Load the certificate.
    // Your plugin will probably be more intelligent and pick up a specific certificate for the user.
    vector<char> certificateFileContent;
    std::string certificateFilePath = GetHomeDirectory() + "/UserIdentifyingCertificate.cer";
    TRACE("Loading certificate data from file");
    if (!LoadFileIntoVector(certificateFilePath, certificateFileContent))
    {
        // Could not load the file.
        // This protocol is not suitable. (Or this might be considered a SystemError)
        // You can configure in the registry whether to fallback to other authentication types when this result is returned.
        TRACE("The file could not be loaded");

        return UnsuitableProtocol;
    }
    TRACE_WITH_ARGS("File data loaded (%zu bytes)", certificateFileContent.size());

    // Try posting the token request and data to StoreFront.
    TRACE("Making HTTP POST to StoreFront");

    PostCertificateAndTokenRequest httpRequest;
    vector<char> authenticationResponseData;
    eAuthResult result = httpRequest.PostRequestAndCertificateToAuthExtension(startURL,
                                                                              tokenRequest,
                                                                              certificateFileContent,
                                                                              /* out */ authenticationResponseData);
    if (result == Success)
    {
        // Allocate the memory to pass out of this function and copy the authentication data to it.
        *outAuthenticationData = (UTF8ByteArray)malloc(authenticationResponseData.size());

        if (*outAuthenticationData == NULL)
        {
            // Out of memory.
            TRACE("A memory allocation error occurred");

            return SystemError;
        }
        memcpy(*outAuthenticationData, &authenticationResponseData[0], authenticationResponseData.size());
        *outDataSize = authenticationResponseData.size();
    }

    TRACE_WITH_ARGS("Authenticate result is %u", result);

    return result;
}

// Call this to free the data returned by Authenticate.
void CitrixAuthenticationManager_Free(void* authenticationData)
{
    TRACE("Plugin's Free function called");
    if (authenticationData != NULL)
    {
        free(authenticationData);
    }
}

} // namespace CitrixAuthManagerCustomProtocolSDK
